package com.xxfc.platform.order.biz;


import com.github.wxiaoqi.security.admin.entity.AppUserLogin;
import com.github.wxiaoqi.security.admin.feign.UserFeign;
import com.github.wxiaoqi.security.common.biz.BaseBiz;
import com.github.wxiaoqi.security.common.vo.PageDataVO;
import com.xxfc.platform.order.contant.enumerate.OrderStatusEnum;
import com.xxfc.platform.order.entity.OrderActivityDetail;;
import com.xxfc.platform.order.mapper.OrderActivityDetailMapper;
import com.xxfc.platform.order.pojo.dto.ActivityOrderFindDTO;
import com.xxfc.platform.order.pojo.dto.MemberOrderBo;
import com.xxfc.platform.order.pojo.vo.OrderActivityInfoStaticVo;
import com.xxfc.platform.order.pojo.vo.OrderActivityInfoVo;
import com.xxfc.platform.order.service.OrderMemberService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import tk.mybatis.mapper.entity.Example;

import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
@Api("活动订单详情")
public class OrderActivityInfoDetailBiz extends BaseBiz<OrderActivityDetailMapper, OrderActivityDetail> {


    @Autowired
    UserFeign userFeign;

    @Autowired
    OrderMemberService orderMemberService;

    private SimpleDateFormat simleFormatter = new SimpleDateFormat("YYYY-MM-dd HH:mm:ss");


    public  OrderActivityDetail getOne(Integer orderId) {
        Example example = new Example(OrderActivityDetail.class);
        example.createCriteria().andEqualTo("orderId", orderId);
        return mapper.selectOneByExample(example);
    }


    public List<OrderActivityInfoVo> getList(ActivityOrderFindDTO activityOrderFindDTO){
        return  mapper.selectList(activityOrderFindDTO);
    }


    public OrderActivityInfoVo getInfo(ActivityOrderFindDTO activityOrderFindDTO){
        List<OrderActivityInfoVo> list = mapper.selectList(activityOrderFindDTO);
        if (list.size() > 0){
            return list.get(0);
        }
        return  null;
    }



    public PageDataVO<OrderActivityInfoVo> selectList(ActivityOrderFindDTO activityOrderFindDTO) {
        PageDataVO<OrderActivityInfoVo> dataVO = new PageDataVO<>();
        dataVO.setPageNum(activityOrderFindDTO.getPage());
        dataVO.setPageSize(activityOrderFindDTO.getLimit());
        dataVO.setData(Collections.EMPTY_LIST);
        List<AppUserLogin> parentUserLogins = null;
        if (StringUtils.isNotEmpty(activityOrderFindDTO.getLeaderPhone())) {
            parentUserLogins = userFeign.getOne(activityOrderFindDTO.getLeaderPhone(),null);
            if (parentUserLogins  ==  null || parentUserLogins.size() == 0 )
                return dataVO;
            List<Integer> userIds = parentUserLogins.stream().map(AppUserLogin::getId).distinct().collect(Collectors.toList());
            activityOrderFindDTO.setParentUserIds(userIds);
        }
        List<AppUserLogin> userLogins = null;
        if (StringUtils.isNotEmpty(activityOrderFindDTO.getName()) || StringUtils.isNotEmpty(activityOrderFindDTO.getPhone())) {
            userLogins = userFeign.getOne(activityOrderFindDTO.getPhone(),activityOrderFindDTO.getName());
            if (userLogins  ==  null || userLogins.size() == 0 )
                return dataVO;
            List<Integer> userIds = userLogins.stream().map(AppUserLogin::getId).distinct().collect(Collectors.toList());
            activityOrderFindDTO.setUserIds(userIds);
        }
        dataVO = PageDataVO.pageInfo(activityOrderFindDTO.getPage(), activityOrderFindDTO.getLimit(), () -> getList(activityOrderFindDTO));
        List<OrderActivityInfoVo> data = dataVO.getData();
        if (CollectionUtils.isEmpty(data)) {
            return dataVO;
        }
        Map<Integer, AppUserLogin> parentUserLoginMap = new HashMap<>();
        if (parentUserLogins == null ) {
            List<Integer> userIds = data.stream().map(OrderActivityInfoVo::getParentUserId).distinct().collect(Collectors.toList());
            parentUserLoginMap = userFeign.findAppUsersByUserIds(userIds);
        }else {
            for (AppUserLogin appUserLogin : parentUserLogins){
                parentUserLoginMap.put(appUserLogin.getId(),appUserLogin);
            }
        }

        Map<Integer, AppUserLogin> userLoginMap = new HashMap<>();
        if (userLogins == null ) {
            List<Integer> userIds = data.stream().map(OrderActivityInfoVo::getUserId).distinct().collect(Collectors.toList());
            userLoginMap = userFeign.findAppUsersByUserIds(userIds);
        }else {
            for (AppUserLogin appUserLogin : userLogins){
                userLoginMap.put(appUserLogin.getId(),appUserLogin);
            }
        }

        for (OrderActivityInfoVo orderActivityInfoVo : data) {
             if (orderActivityInfoVo.getParentUserId() != null ){
                 AppUserLogin userLogin = parentUserLoginMap.get(orderActivityInfoVo.getParentUserId());
                 if (userLogin != null ){
                     orderActivityInfoVo.setParentPhone(userLogin.getUsername());
                     orderActivityInfoVo.setParentName(userLogin.getRealname());
                 }
             }
             AppUserLogin userLogin = userLoginMap.get(orderActivityInfoVo.getUserId());
             if (userLogin != null ){
                 orderActivityInfoVo.setPhone(userLogin.getUsername());
                 orderActivityInfoVo.setRealName(userLogin.getRealname());
             }
        }
        return dataVO;
    }


    public OrderActivityInfoStaticVo selectListStatic(ActivityOrderFindDTO activityOrderFindDTO) {
        OrderActivityInfoStaticVo orderActivityInfoStaticVo = new OrderActivityInfoStaticVo();
        List<AppUserLogin> parentUserLogins = null;
        if (StringUtils.isNotEmpty(activityOrderFindDTO.getLeaderPhone())) {
            parentUserLogins = userFeign.getOne(activityOrderFindDTO.getLeaderPhone(),null);
            if (parentUserLogins  ==  null || parentUserLogins.size() == 0 )
                return orderActivityInfoStaticVo;
            List<Integer> userIds = parentUserLogins.stream().map(AppUserLogin::getId).distinct().collect(Collectors.toList());
            activityOrderFindDTO.setParentUserIds(userIds);
        }
        List<AppUserLogin> userLogins = null;
        if (StringUtils.isNotEmpty(activityOrderFindDTO.getName()) || StringUtils.isNotEmpty(activityOrderFindDTO.getPhone())) {
            userLogins = userFeign.getOne(activityOrderFindDTO.getPhone(),activityOrderFindDTO.getName());
            if (userLogins  ==  null || userLogins.size() == 0 )
                return orderActivityInfoStaticVo;
            List<Integer> userIds = userLogins.stream().map(AppUserLogin::getId).distinct().collect(Collectors.toList());
            activityOrderFindDTO.setUserIds(userIds);
        }
        orderActivityInfoStaticVo = getListStatic(activityOrderFindDTO);
        return orderActivityInfoStaticVo;
    }


    public  OrderActivityInfoStaticVo  getListStatic(ActivityOrderFindDTO activityOrderFindDTO){
        return  mapper.selectListStatic(activityOrderFindDTO);
    }



    /**
     * 导出数据
     * @param
     */
    public void exportMemberOrderData(ActivityOrderFindDTO activityOrderFindDTO, String name, OutputStream outputStream) throws Exception{
        PageDataVO<OrderActivityInfoVo> dataVO = selectList(activityOrderFindDTO);
        List<OrderActivityInfoVo> memberOrderBoList = dataVO.getData();
        activityOrderFindDTO.setStatus(OrderStatusEnum.ORDER_FINISH.getCode());
        OrderActivityInfoStaticVo orderActivityInfoStaticVo = selectListStatic(activityOrderFindDTO);

        XSSFWorkbook hssfWorkbook = new XSSFWorkbook();
        XSSFSheet sheet = hssfWorkbook.createSheet(name+"活动订单列表");
        CellStyle headerCellStyle = orderMemberService.createHeaderCellStyle(hssfWorkbook);
        CellStyle generalCellStyle = orderMemberService.createGeneralCellStyle(hssfWorkbook);

        Row row1 = sheet.createRow(0);
        orderMemberService.createHeader(row1,0,new String[]{name+"活动订单列表"},headerCellStyle);
        CellRangeAddress cellRangeAddress = new CellRangeAddress(0,0,0,11);
        sheet.addMergedRegion(cellRangeAddress);

        Row row2 = sheet.createRow(1);
        Cell row2_cell1 = row2.createCell(0);
        row2_cell1.setCellValue("统计");
        row2_cell1.setCellStyle(headerCellStyle);
        CellRangeAddress cellRangeAddress2 = new CellRangeAddress(1,2,0,0);
        sheet.addMergedRegion(cellRangeAddress2);
        String[] header2 = new String[]{"活动订单总数(单)","活动订单总额(元)","活动佣金总额"};
        orderMemberService.createHeader(row2,1,header2,headerCellStyle);
        String[] countData = getData(orderActivityInfoStaticVo);
        Row row4 = sheet.createRow(2);
        orderMemberService.createCellCount(row4,1,countData,generalCellStyle);

        Row row5 = sheet.createRow(4);
        String[] header3 = new String[]{"序号","名称","订单号","创建时间","用户姓名","手机号","订单价格(元)","上级姓名/手机号","拥金(元)","支付时间","订单状态"};
        CellRangeAddress cellRangeAddress3 = new CellRangeAddress(3,3,0,header3.length);
        sheet.addMergedRegion(cellRangeAddress3);
        orderMemberService.createHeader(row5,0,header3,headerCellStyle);

        for(int i=0;i<header3.length;i++){
            sheet.setColumnWidth(i,24*256);
        }
        List<String[]> orderDataList = getMemberOrderDataList(memberOrderBoList, 11);
        orderMemberService.createMemberOrderCellData(sheet,5,generalCellStyle,orderDataList);
        hssfWorkbook.write(outputStream);
        hssfWorkbook.close();

    }

    private List<String[]> getMemberOrderDataList(List<OrderActivityInfoVo> memberOrderBoList,Integer size){
        List<String[]> memerOrderData = new ArrayList<>();
        String[] data;
        int i = 0;
        for (OrderActivityInfoVo activityInfoVo : memberOrderBoList) {
            data = new String[size];
            i++;
            data[0]=i+"";
            data[1]=activityInfoVo.getName();
            data[2]=activityInfoVo.getNo();
            data[3]=simleFormatter.format(activityInfoVo.getCrtTime());
            data[4]=activityInfoVo.getRealName();
            data[5]=activityInfoVo.getPhone();
            data[6]=activityInfoVo.getRealAmount().toString();
            data[7]=StringUtils.isNotBlank(activityInfoVo.getParentPhone())  ? activityInfoVo.getParentName()+"/"+ activityInfoVo.getParentPhone(): "";
            data[8]=activityInfoVo.getParentComisson().toString();
            data[9]=activityInfoVo.getPayTime() != null ? simleFormatter.format(activityInfoVo.getPayTime()) : "";
            data[10]=activityInfoVo.getStatus()==2?"取消":activityInfoVo.getStatus()==3?"待支付":activityInfoVo.getStatus()==6?"已完成":"";
            memerOrderData.add(data);
        }
        return memerOrderData;
    }


    public String[] getData(OrderActivityInfoStaticVo orderActivityInfoStaticVo) {
        String[] strings = new String[3];
        strings[0] = String.valueOf(orderActivityInfoStaticVo.getNumber());
        strings[1] = String.valueOf(orderActivityInfoStaticVo.getTotalAmount());
        strings[2] = String.valueOf(orderActivityInfoStaticVo.getTotalComissonAmount());
        return strings;
    }



}
